unit DBBackupOptionSetting;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, FileCtrl, Mask, DBCtrls, wwdbdatetimepicker, MyConst;

type
  TfrmDBBackupOptionSetting = class(TForm)
    btnOK: TButton;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label3: TLabel;
    btnOpenDB: TButton;
    DirectoryLB: TDirectoryListBox;
    GroupBox2: TGroupBox;
    Label4: TLabel;
    Label2: TLabel;
    GroupBox3: TGroupBox;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    btnCancel: TButton;
    OpenDBDialog: TOpenDialog;
    OriEdtPath: TDBEdit;
    BupEdtPath: TDBEdit;
    btnOpenBackup: TButton;
    cbFormat: TComboBox;
    cbMetadata: TComboBox;
    cbGarbage: TComboBox;
    cbLimbo: TComboBox;
    cbCheckSums: TComboBox;
    cbConvert: TComboBox;
    cbVerbose: TComboBox;
    procedure btnCancelClick(Sender: TObject);
    procedure btnOpenDBClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnOpenBackupClick(Sender: TObject);
  private
    { Private declarations }
    Function IsEmptyStringComponent(lblCaption : String; Component : TComponent) : Boolean;
    procedure PostToDBOptionSetting;
    procedure Prepare_Option;
  public
    { Public declarations }
  end;

var
  frmDBBackupOptionSetting: TfrmDBBackupOptionSetting;

implementation

uses DatGPro;

{$R *.DFM}

Function TfrmDBBackupOptionSetting.IsEmptyStringComponent(lblCaption : String; Component : TComponent) : Boolean;
  Function IsEmptyString(vEmptyString : String) : Boolean;
  begin
    Result := False;
    if Length(Trim(vEmptyString)) = 0 then
      Result := True;
  end;
  
begin
  Result := False;

  If Component is TEdit then
    Begin
      If IsEmptyString(TEdit(Component).Text) then
      Begin
        TEdit(Component).Text := '';
        If TEdit(Component).Enabled then
          TEdit(Component).SetFocus;
        MessageDlg('Please Key in ' + lblCaption, mtWarning, [mbOK], 0);
        Result := True;
      End;
    End
  Else If Component is TComboBox then
    Begin
      If IsEmptyString(TComboBox(Component).Text) then
      Begin
        If TComboBox(Component).Enabled then
          TComboBox(Component).SetFocus;
        MessageDlg('Please Key in ' + lblCaption, mtWarning, [mbOK], 0);
        Result := True;
      End;
    End
  Else If Component is TDBEdit then
    Begin
      If IsEmptyString(TDBEdit(Component).Text) then
      Begin
        TDBEdit(Component).Text := '';
        If TDBEdit(Component).Enabled then
          TDBEdit(Component).SetFocus;
        MessageDlg('Please Key in ' + lblCaption, mtWarning, [mbOK], 0);
        Result := True;
      End;
    End
  Else If Component is TDBComboBox then
    Begin
      If IsEmptyString(TDBComboBox(Component).Text) then
      Begin
        If TDBComboBox(Component).Enabled then
          TDBComboBox(Component).SetFocus;
        MessageDlg('Please Key in ' + lblCaption, mtWarning, [mbOK], 0);
        Result := True;
      End;
    End
  Else If Component is TwwDBDateTimePicker then
    Begin
      If (TwwDBDateTimePicker(Component).DateTime <= 0) then
      Begin
        If TwwDBDateTimePicker(Component).Enabled then
          TwwDBDateTimePicker(Component).SetFocus;
        MessageDlg('Please Key in ' + lblCaption, mtWarning, [mbOK], 0);
        Result := False;
      End
    End;
end;

procedure TfrmDBBackupOptionSetting.PostToDBOptionSetting;
var
  path : String;
begin
  path := ExtractFilePath(Application.ExeName);

  With DmGpro.cdsDBOptionSetting do
  Begin
    FieldByName('ServerAlias').AsString        := Trim(OriEdtPath.Text);
    FieldByName('BackupAlias').AsString        := Trim(BupEdtPath.Text);
    FieldByName('Format').AsString             := Trim(cbFormat.Text);
    FieldByName('MetadataOnly').AsString       := Trim(cbMetadata.Text);
    FieldByName('GarbageCollection').AsString  := Trim(cbGarbage.Text);
    FieldByName('TransactionInLimbo').AsString := Trim(cbLimbo.Text);
    FieldByName('Checksums').AsString          := Trim(cbCheckSums.Text);
    FieldByName('ConvertToTables').AsString    := Trim(cbConvert.Text);
    FieldByName('VerboseOutput').AsString      := Trim(cbVerbose.Text);
    If (FieldByName('AutoTerminate').AsString = '') then
      FieldByName('AutoTerminate').AsString := 'OFF';
    Post;

    SaveToFile(path + Dir_CDS + File_OptionSetting);
  End;
end;

procedure TfrmDBBackupOptionSetting.Prepare_Option;
begin
  cbFormat.Clear;
  cbFormat.Items.Add('Non-Transportable');
  cbFormat.Items.Add('Transportable');

  cbMetadata.Clear;
  cbMetadata.Items.Add('False');
  cbMetadata.Items.Add('True');

  cbGarbage.Clear;
  cbGarbage.Items.Add('False');
  cbGarbage.Items.Add('True');

  cbLimbo.Clear;
  cbLimbo.Items.Add('Ignore');
  cbLimbo.Items.Add('Process');

  cbCheckSums.Clear;
  cbCheckSums.Items.Add('Ignore');
  cbCheckSums.Items.Add('Process');

  cbConvert.Clear;
  cbConvert.Items.Add('False');
  cbConvert.Items.Add('True');

  cbVerbose.Clear;
  cbVerbose.Items.Add('None');
  cbVerbose.Items.Add('To File');
  cbVerbose.Items.Add('To Screen');

  cbFormat.ItemIndex    := 1;
  cbMetadata.ItemIndex  := 0;
  cbGarbage.ItemIndex   := 1;
  cbLimbo.ItemIndex     := 1;
  cbCheckSums.ItemIndex := 1;
  cbConvert.ItemIndex   := 0;
  cbVerbose.ItemIndex   := 2;
end;

procedure TfrmDBBackupOptionSetting.btnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmDBBackupOptionSetting.btnOpenDBClick(Sender: TObject);
begin
  {Database Alias}
  OpenDBDialog.InitialDir := DirectoryLB.Directory;
  OpenDBDialog.Title := 'Select Database';
  OpenDBDialog.Filter := 'Database File (*.ib; *.gdb)|*.ib;*.gdb|All files (*.*)|*.*';
  If OpenDBDialog.Execute then
    OriEdtPath.Text := UpperCase(OpenDBDialog.FileName);
end;

procedure TfrmDBBackupOptionSetting.btnOpenBackupClick(Sender: TObject);
begin
  {Backup File Alias}
  OpenDBDialog.InitialDir := DirectoryLB.Directory;
  OpenDBDialog.Title := 'Select Backup File';
  OpenDBDialog.Filter := 'Database File (*.gbk)|*.gbk|All files (*.*)|*.*';
  If OpenDBDialog.Execute then
    BupEdtPath.Text := UpperCase(OpenDBDialog.FileName);
end;

procedure TfrmDBBackupOptionSetting.FormShow(Sender: TObject);
begin
  Prepare_Option;
  OriEdtPath.SetFocus;

  DmGpro.cdsDBOptionSetting.Edit;
end;

procedure TfrmDBBackupOptionSetting.btnOKClick(Sender: TObject);
var
  FileType : String;
begin
  If (Pos(' ', Trim(OriEdtPath.Text)) > 0) then
  Begin
    MessageDlg('Database alias cannot have space key', mtWarning, [mbOK], 0);
    OriEdtPath.SetFocus;
    Exit;
  End;

  If (Pos(' ', Trim(BupEdtPath.Text)) > 0) then
  Begin
    MessageDlg('Backup file alias cannot have space key', mtWarning, [mbOK], 0);
    BupEdtPath.SetFocus;
    Exit;
  End;

  If IsEmptyStringComponent('Database Alias', OriEdtPath) then
  Begin
    OriEdtPath.SetFocus;
    Exit;
  End;

  If IsEmptyStringComponent('Backup File Alias', BupEdtPath) then
  Begin
    BupEdtPath.SetFocus;
    Exit;
  End;

  FileType := Copy(Trim(BupEdtPath.Text), Length(Trim(BupEdtPath.Text)) -2, 4);
  If (Trim(Uppercase(FileType)) <> 'GBK') then
  begin
    MessageDlg('Please type a backup file (.GBK) before proceed', mtWarning, [mbok], 0);
    BupEdtPath.SetFocus;
    Exit;
  end;

  PostToDBOptionSetting;
  Close;

  ModalResult := mrOk;
end;

end.
