unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ComCtrls, ToolWin, Buttons, ImgList, StdCtrls, ExtCtrls, IBServices,
  MyConst, FileCtrl, Validate;

type
  TfrmMain = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Edit: TMenuItem;
    DBOpstionSetting: TMenuItem;
    N2: TMenuItem;
    Exit1: TMenuItem;
    SaveAs: TMenuItem;
    Copy: TMenuItem;
    SelectAll: TMenuItem;
    N3: TMenuItem;
    Find: TMenuItem;
    StatusBar1: TStatusBar;
    ImageList1: TImageList;
    ToolBar1: TToolBar;
    tbDBOpstionSetting: TToolButton;
    ToolButton2: TToolButton;
    tbSave: TToolButton;
    tmrCountDown: TTimer;
    SaveLogFileDialog: TSaveDialog;
    tbFind: TToolButton;
    FindDialog1: TFindDialog;
    RichEdit1: TRichEdit;
    tbExit: TToolButton;
    tmrOptionSetting: TTimer;
    DirectoryLB: TDirectoryListBox;
    AutoTerminate: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure DBOpstionSettingClick(Sender: TObject);
    procedure tmrCountDownTimer(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SelectAllClick(Sender: TObject);
    procedure CopyClick(Sender: TObject);
    procedure SaveAsClick(Sender: TObject);
    procedure tbFindClick(Sender: TObject);
    procedure FindDialog1Find(Sender: TObject);
    procedure RichEdit1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure RichEdit1KeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure RichEdit1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FindClick(Sender: TObject);
    procedure tbExitClick(Sender: TObject);
    procedure tbSaveClick(Sender: TObject);
    procedure tbDBOpstionSettingClick(Sender: TObject);
    procedure tmrOptionSettingTimer(Sender: TObject);
    procedure AutoTerminateClick(Sender: TObject);
  private
    { Private declarations }
    sSec : Integer;
    Min, Sec : Integer;

    FServer    : String;
    FBackup    : String;
    FFormat    : String;
    FMeta      : String;
    FGarbage   : String;
    FLimbo     : String;
    FChecksums : String;
    FConvert   : String;
    FVerbose   : String;
    FUser      : String;
    FPassword  : String;

    bValidation : Boolean;
    bComplete : Boolean;

    procedure DecodeSecurity(pUser, pPassword : String);
    procedure CheckDBOptionSetting;
    procedure PrepareTime(pSec : Integer);
    procedure MouseCoordinate;
    procedure StatusMessage(pTextColumn : Integer; pMessage : String);
    procedure BackupCountDown(pMin, pSec : Integer; pStatus : String);
    procedure Prepare_ServerParam;
    procedure ValidationDatabase;
    procedure StartDatabaseBackup;
    procedure TerminateDatabaseBackup;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses DBBackupOptionSetting, DatGPro, AutoTerminate;

{$R *.DFM}

procedure TfrmMain.DecodeSecurity(pUser, pPassword : String);
begin
  FUser := FDecode(pUser);
  FPassword := FDecode(pPassword);
end;

procedure TfrmMain.CheckDBOptionSetting;
begin
  RichEdit1.PlainText := True;
  RichEdit1.Clear;

  If (DmGPro.cdsDBOptionSetting.RecordCount = 0) then
    Begin
      PrepareTime(3);
      BackupCountDown(Min, Sec, Status_Setting);
      tmrOptionSetting.Enabled := True;
      tmrCountDown.Enabled := False;
    End
  Else
    Begin
      PrepareTime(60);
      bValidation := False;
      bComplete := False;
      BackupCountDown(Min, Sec, Status_Progress);
      tmrCountDown.Enabled := True;
      tmrOptionSetting.Enabled := False;
    End;
end;

procedure TfrmMain.PrepareTime(pSec : Integer);
begin
  Min := 1;
  Sec  := 0;
  sSec := pSec;
end;

procedure TfrmMain.MouseCoordinate;
var
  BeginInt, IntX, IntY : Integer;
begin
  BeginInt := 1;
  IntY := SendMessage(RichEdit1.Handle, EM_LINEFROMCHAR, RichEdit1.SelStart,0);
  IntX := RichEdit1.SelStart - SendMessage(RichEdit1.Handle, EM_LINEINDEX, IntY, 0);

  StatusMessage(0, '     ' + IntToStr(IntY + BeginInt) +':  '+ IntToStr(IntX + BeginInt));
end;

procedure TfrmMain.StatusMessage(pTextColumn : Integer; pMessage : String);
begin
  If pTextColumn = 0 then
    StatusBar1.Panels[0].Text := pMessage;

  StatusBar1.Panels[1].Text := 'Read-Only';

  If pTextColumn = 2 then
    StatusBar1.Panels[2].Text := pMessage;
end;

procedure TfrmMain.BackupCountDown(pMin, pSec : Integer; pStatus : String);
begin
  If (pStatus = Status_Progress) then
  Begin
    If ((pMin = 0) and (pSec = 0)) then
      StatusMessage(2, 'Database backup in progress ...')
    Else
      StatusMessage(2, 'Database backup starting in' +' '+ IntToStr(pMin) +':'+ IntToStr(pSec));
  End;

  If (pStatus = Status_Terminate) then
  Begin
    If not ((pMin = 0) and (pSec = 0)) then
      StatusMessage(2, 'Application terminate in' +' '+ IntToStr(pMin) +':'+ IntToStr(pSec));
  End;

  If (pStatus = Status_Setting) then
  Begin
    StatusMessage(2, 'Cannot find database backup option setting');
  End;
end;

procedure TfrmMain.Prepare_ServerParam;
begin
  FServer    := Trim(DmGPro.cdsDBOptionSetting.FieldByName('ServerAlias').AsString);
  FBackup    := Trim(DmGPro.cdsDBOptionSetting.FieldByName('BackupAlias').AsString);
  FFormat    := Trim(DmGPro.cdsDBOptionSetting.FieldByName('Format').AsString);
  FMeta      := Trim(DmGPro.cdsDBOptionSetting.FieldByName('MetadataOnly').AsString);
  FGarbage   := Trim(DmGPro.cdsDBOptionSetting.FieldByName('GarbageCollection').AsString);
  FLimbo     := Trim(DmGPro.cdsDBOptionSetting.FieldByName('TransactionInLimbo').AsString);
  FChecksums := Trim(DmGPro.cdsDBOptionSetting.FieldByName('Checksums').AsString);
  FConvert   := Trim(DmGPro.cdsDBOptionSetting.FieldByName('ConvertToTables').AsString);
  FVerbose   := Trim(DmGPro.cdsDBOptionSetting.FieldByName('VerboseOutput').AsString);

  DecodeSecurity(Sys_UserName, Sys_Password);
end;

procedure TfrmMain.ValidationDatabase;
begin
  Screen.Cursor    := crHourGlass;

  With DmGPro.IBValidationService1 do
  Begin
    Try
      ServerName := '127.0.0.1';
      DatabaseName := FServer;
      LoginPrompt := False;
      Params.Add('user_name='+FUser);
      Params.Add('password='+FPassword);
      Active := True;
    Except
      Messagedlg('Sysdba password had change, please choose another process to proceed!', mtWarning, [mbok], 0);
      Screen.Cursor := crDefault;
      Exit;
    End;

    Options := [];
    Try
      //Options := Options + [LimboTransactions];
      Options := Options  + [CheckDB];
      //Options := Options + [IgnoreChecksum];
      //Options := Options + [KillShadows];
      //Options := Options + [MendDB];
      //Options := Options + [SweepDB];
      Options := Options + [ValidateDB];
      Options := Options + [ValidateFull];
    Finally
    End;

    Try
      ServiceStart;
      While IsServiceRunning do 
      Begin
        bValidation := True;
      End;

      RichEdit1.Lines.Add('Database validation completed. No error found.');
      RichEdit1.Lines.Add('');
    Except
       ON E:Exception Do
         Begin
           bValidation := False;
           RichEdit1.lines.add(E.Message);
         End;
    End; //try

    Active := False;
  End; //End Of With DmGPro.IBValidationService_GPro

  Screen.Cursor := crDefault;
end;

procedure TfrmMain.StartDatabaseBackup;
var
  path, LogFile, LogDate : String;
  vServiceTime : TDateTime;

begin
  Screen.Cursor := crHourGlass;

  With DmGPro.IBBackupService1 do
  Begin
    BackupFile.Clear;
    Try
      Protocol := Local;
      ServerName := Sys_Server;
      LoginPrompt := False;
      Params.Add('user_name='+FUser);
      Params.Add('password='+FPassword);
      Active := True;
    Except
      MessageDlg('Sysdba password had change, please choose another process to proceed', mtWarning, [mbok], 0);
      bComplete := False;
      Screen.cursor := crDefault;
      Exit;
    End;

    Try
      Options := [];
      If ((FVerbose = 'To File') or (FVerbose = 'To Screen')) then
         Verbose := True;
      If (FFormat = 'Non-Transportable') then
         Options := [NonTransportable];
      If (FMeta = 'True') then
         Options := Options + [MetadataOnly];
      If (FGarbage = 'False') then
         Options := Options + [NoGarbageCollection];
      If (FLimbo = 'Ignore') then
         Options := Options + [IgnoreLimbo];
      If (FChecksums = 'Ignore') then
         Options := Options + [IgnoreChecksums];
      If (FConvert = 'True') then
         Options := Options + [ConvertExtTables];

      DatabaseName := FServer;
      BackupFile.Add(FBackup);
      Application.ProcessMessages;
      ServiceStart;

      vServiceTime := Now;
      RichEdit1.Lines.Add(FormatDateTime('"Service started at " mm/dd/yyyy hh:mm:ss AM/PM', vServiceTime));
      RichEdit1.Lines.Add('');
      While not Eof do
      begin
        RichEdit1.Lines.Add(GetNextLine);
        Application.ProcessMessages;
      end;
    Finally
      Active := False;
      vServiceTime := Now;
      RichEdit1.Lines.Add(FormatDateTime('"Service ended at " mm/dd/yyyy hh:mm:ss AM/PM', vServiceTime));
      StatusMessage(2, 'Database backup completed');

      path    := ExtractFilePath(Application.ExeName) + Dir_Log;
      LogDate := FormatDateTime('yyyymmdd', Now);
      LogFile := File_GBKLog + LogDate + '.txt';

      RichEdit1.Lines.SaveToFile(path + LogFile);

      bComplete := True;
    End;
  End; //End Of With DmGPro.IBBackupService_GPro

  Screen.Cursor := crDefault;

  { After 10 sec , Terminate the application }
  If ((bComplete) and (DmGpro.cdsDBOptionSetting.FieldByName('AutoTerminate').AsString = 'ON')) then
  Begin
    PrepareTime(10);
    tmrCountDown.Enabled := True;
  End;
end;

procedure TfrmMain.TerminateDatabaseBackup;
begin
  Application.Terminate;
end;

procedure TfrmMain.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.DBOpstionSettingClick(Sender: TObject);
begin
  tmrCountDown.Enabled := False;

  Try
    frmDBBackupOptionSetting := TfrmDBBackupOptionSetting.Create(Application);
    frmDBBackupOptionSetting.ShowModal;
  Finally
    frmDBBackupOptionSetting.Release;
  End;

  CheckDBOptionSetting;
end;

procedure TfrmMain.tmrCountDownTimer(Sender: TObject);
var
  I : Integer;
begin
  sSec := sSec - 1;

  Min := 0;
  Sec := sSec;

  If not bComplete then
    Begin
      BackupCountDown(Min, Sec, Status_Progress);

      If ((Min = 0) and (Sec = 0)) then
      Begin
        tmrCountDown.Enabled := False;
        Prepare_ServerParam;

        For I := 1 to 3 do begin
          If Not bValidation then
            ValidationDatabase
          Else
            break;
        End;

        StartDatabaseBackup;
      End;
    End
  Else
    Begin
      BackupCountDown(Min, Sec, Status_Terminate);

      If ((Min = 0) and (Sec = 0)) then
      Begin
        tmrCountDown.Enabled := False;
        TerminateDatabaseBackup
      End;
    End;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  CheckDBOptionSetting;
  MouseCoordinate;
end;

procedure TfrmMain.SelectAllClick(Sender: TObject);
begin
  RichEdit1.SelectAll;
end;

procedure TfrmMain.CopyClick(Sender: TObject);
begin
  RichEdit1.CopyToClipboard;
end;

procedure TfrmMain.SaveAsClick(Sender: TObject);
var
  path : String;
begin
  SaveLogFileDialog.InitialDir := DirectoryLB.Directory;
  SaveLogFileDialog.Title := 'Save As';
  SaveLogFileDialog.Filter := 'Text Documents (*.txt)|*.txt|All files|*.*';
  If SaveLogFileDialog.Execute then
  Begin
    path := SaveLogFileDialog.FileName;
    RichEdit1.Lines.SaveToFile(path + '.txt');
  End;  
end;

procedure TfrmMain.tbFindClick(Sender: TObject);
begin
  FindClick(Sender);
end;

procedure TfrmMain.FindDialog1Find(Sender: TObject);
var
  FoundAt, nStartPos, nToEnd : Integer;
  search : Integer;
  FindText : String;
begin
  FindText := FindDialog1.FindText;

  If (RichEdit1.SelLength > 0) then
    nStartPos := RichEdit1.SelStart + RichEdit1.SelLength
  Else
    nStartPos := 0;

  nToEnd := Length(RichEdit1.Text) - nStartPos;

  { Find down }
  If (frDown in FindDialog1.Options) then
  Begin
    If (frMatchCase in FindDialog1.Options) then
      FoundAt := RichEdit1.FindText(FindText, nStartPos, nToEnd, [stMatchCase])
    Else If (frWholeWord in FindDialog1.Options) then
      FoundAt := RichEdit1.FindText(FindText, nStartPos, nToEnd, [stWholeWord])
    Else If ((frMatchCase in FindDialog1.Options) and (frWholeWord in FindDialog1.Options)) then
      FoundAt := RichEdit1.FindText(FindText, nStartPos, nToEnd, [stMatchCase, stWholeWord])
    Else
      FoundAt := RichEdit1.FindText(FindText, nStartPos, nToEnd, [])

  End;

  { Find Up }
  If not (frDown in FindDialog1.Options) then
  Begin
    search := -1 ;
    Repeat
      FoundAt := search;

      If (frMatchCase in FindDialog1.Options) then
        search := RichEdit1.FindText(FindText, search + 1, RichEdit1.SelStart - search -1, [stMatchCase])
      Else If (frWholeWord in FindDialog1.Options) then
        search := RichEdit1.FindText(FindText, search + 1, RichEdit1.SelStart - search -1, [stWholeWord])
      Else If ((frMatchCase in FindDialog1.Options) and (frWholeWord in FindDialog1.Options)) then
        search := RichEdit1.FindText(FindText, search + 1, RichEdit1.SelStart - search -1, [stMatchCase, stWholeWord])
      Else
        search := RichEdit1.FindText(FindText, search + 1, RichEdit1.SelStart - search -1, [])

    Until
      search < 0;
  End;

  If (FoundAt <> -1) then
    Begin
      RichEdit1.SetFocus;
      RichEdit1.SelStart := FoundAt;
      RichEdit1.SelLength := Length(FindText);
      FindDialog1.Execute;
    end
  Else
    MessageDlg('Cannot find ' +'"'+ FindText +'"', mtInformation, [mbok], 0);
end;

procedure TfrmMain.RichEdit1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  MouseCoordinate;
end;

procedure TfrmMain.RichEdit1KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  MouseCoordinate;
end;

procedure TfrmMain.RichEdit1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MouseCoordinate;
end;

procedure TfrmMain.FindClick(Sender: TObject);
begin
  FindDialog1.Execute
end;

procedure TfrmMain.tbExitClick(Sender: TObject);
begin
  Exit1Click(Sender);
end;

procedure TfrmMain.tbSaveClick(Sender: TObject);
begin
  SaveAsClick(Sender);
end;

procedure TfrmMain.tbDBOpstionSettingClick(Sender: TObject);
begin
  DBOpstionSettingClick(Sender);
end;

procedure TfrmMain.tmrOptionSettingTimer(Sender: TObject);
begin
  sSec := sSec - 1;

  Min := 0;
  Sec := sSec;

  If ((Min = 0) and (Sec = 0)) then
  Begin
    tmrOptionSetting.Enabled := False;
    DBOpstionSettingClick(Sender);
  End;
end;

procedure TfrmMain.AutoTerminateClick(Sender: TObject);
begin
  tmrCountDown.Enabled := False;

  Try
    frmAutoTerminate := TfrmAutoTerminate.Create(Application);
    frmAutoTerminate.ShowModal;
  Finally
    frmAutoTerminate.Release;
  End;

  CheckDBOptionSetting;
end;

end.
